using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

public partial class AutoGeneratedColumns : System.Web.UI.Page
{
    protected void Page_Load(object sender, EventArgs e)
    {
        GridView1.DataSource = BuildDataSource();
        GridView1.DataBind();
    }
    private DataTable BuildDataSource()
    {
        DataTable dt = new DataTable();
        dt.Columns.Add("FirstName", typeof(string));
        dt.Columns.Add("LastName", typeof(string));
        dt.Columns.Add("DOB", typeof(DateTime));

        DataRow dr;

        dr = dt.NewRow();
        dr["FirstName"] = "John the first";
        dr["LastName"] = "Doe";
        dr["DOB"] = new DateTime(1981, 8, 8);
        dt.Rows.Add(dr);

        dr = dt.NewRow();
        dr["FirstName"] = "Clark";
        dr["LastName"] = "Kent";
        dr["DOB"] = new DateTime(1975, 11, 23);
        dt.Rows.Add(dr);

        return dt;
    }

    /// <summary>
    /// This method formats the Dates in the Auto-Generated columns inside
    /// the GridViewRow.
    /// </summary>
    /// <param name="gv">The GridView that contains the current row</param>
    /// <param name="gvr">The row to format</param>
    private void FormatDatesInGridView(GridView gv, GridViewRow gvr)
    {
        if (gv.DataSource != null)
        {
            DataTable dt = null;

            if (gv.DataSource is DataView)
            {
                dt = ((DataView)gv.DataSource).Table;
            }
            else
                if (gv.DataSource is DataSet)
                {
                    dt = ((DataSet)gv.DataSource).Tables[0];
                }
                else
                    if (gv.DataSource is DataTable)
                    {
                        dt = (DataTable)gv.DataSource;
                    }

            if (gvr.RowType == DataControlRowType.DataRow)
            {
                foreach (TableCell tc in gvr.Cells)
                {
                    DataControlFieldCell dcfc = (DataControlFieldCell)tc;
                    if (dcfc.ContainingField is AutoGeneratedField)
                    {
                        AutoGeneratedField agf = (AutoGeneratedField)dcfc.ContainingField;
                        if (agf.DataType == typeof(DateTime))
                        {
                            System.Reflection.FieldInfo fi;
                            fi = typeof(BoundField).GetField("_dataFormatString", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);
                            fi.SetValue(agf, "{0:dd/MM/yyyy}");

                            fi = typeof(DataControlField).GetField("_statebag", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);
                            ((StateBag)fi.GetValue(agf))["DataFormatString"] = "{0:dd/MM/yyyy}";

                            System.Reflection.MethodInfo mi;
                            mi = typeof(BoundField).GetMethod("OnFieldChanged", System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance);
                            mi.Invoke(agf, null);
                            ((BoundField)agf).HtmlEncode = false;
                        }

                    }
                }
            }
        }
    }

    protected void GridView1_RowCreated(object sender, GridViewRowEventArgs e)
    {
        if(e.Row.RowType == DataControlRowType.DataRow)
            FormatDatesInGridView(GridView1, e.Row);
    }
}
